#!/usr/bin/env python

# Copyright (c) 2018  University of Houston

import argparse
import json
import logging
import sys
import time
import xmlrpclib

logging.basicConfig(level = logging.INFO)

import geni.aggregate.apis
import geni.aggregate.instageni as IGAM
from geni.aggregate.spec import AMSpec, AMTYPE
import geni.minigcf.config as CFG
import geni.rspec.pg as PG
import geni.rspec.igext
import geni.util

import uhexp.client as XC
import uhgeni.ssh

CFG.HTTP.TIMEOUT = 90
IMAGE = "urn:publicid:IDN+emulab.net+image+emulab-ops//UBUNTU16-64-STD"

def parse_args ():
  parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
  parser.add_argument("--uhexpbranch", dest="uhexpbranch", type=str, default="default")
  parser.add_argument("--uhexpurl", dest="uhexpurl", type=str, default="https://bitbucket.org/uh-netlab/uhexp")
  parser.add_argument("--site", dest="site", default = None)
  parser.add_argument("--site-url", dest="site_url", default = "https://instageni.hawaii.edu:12369/protogeni/xmlrpc/am/2.0")
  parser.add_argument("--slice", dest="slice", type=str)
  parser.add_argument("--project", dest="project", type=str, default=None)
  parser.add_argument("--context-path", dest="context_path", type=str, default=None)
  parser.add_argument("--config-path", dest="config_path", type=str, default="./expconfig")
  parser.add_argument("--delete", dest="delete", action="store_true", default=False)
  parser.add_argument("--image", dest="image", default=IMAGE)
  parser.add_argument("--user", dest="user")

  return parser.parse_args()

def resolveSite (context, opts):
  if not opts.site:
    logging.info("Building site from URL (%s)" % (opts.site_url))
    ams = AMSpec()
    ams.shortname = "ref-exp-site"
    ams.url = opts.site_url
    if opts.site_url.count("protogeni"):
      ams.type = AMTYPE.IG
    elif opts.site_url.count("foam"):
      ams.type = AMTYPE.VTS
    elif opts.site_url.count("orca"):
      ams.type = AMTYPE.EG
    return ams.build()

  sites = geni.util.loadAggregates()
  if opts.site in sites:
    logging.info("Cached site found")
    return sites[opts.site]

  logging.info("Updating Sites from Clearinghouse")
  geni.util.updateAggregates(context, sites)
  if opts.site in sites:
    logging.info("Cached site found")
    return sites[opts.site]

  logging.error("Site %s not found" % (opts.site))
  sys.exit(1)


def buildRequest (opts):
  r = PG.Request()

  client = r.XenVM("client")
  client.ram = 768
  client_intf = client.addInterface("if0")
  client_intf.addAddress(PG.IPv4Address("10.0.1.1", "255.255.255.0"))
  
  server = r.XenVM("server")
  server.ram = 1024
  server_intf = server.addInterface("if0")
  server_intf.addAddress(PG.IPv4Address("10.0.1.3", "255.255.255.0"))

  monitor = r.XenVM("monitor")
  monitor.ram = 768
  monitor_intf = monitor.addInterface("if0")

  lan = r.LAN()
  lan.addInterface(client_intf)
  lan.addInterface(server_intf)
  lan.addInterface(monitor_intf)

  return r

if __name__ == '__main__':
  opts = parse_args()

  context = geni.util.loadContext(path = opts.context_path, key_passphrase=True)
  if opts.project:
    context.project = opts.project

  am = resolveSite(context, opts)

  if opts.delete:
    try:
      logging.info("Deleting existing sliver")
      am.deletesliver(context, opts.slice)
      logging.info("Cloudlab deletes take time, please try to make a new sliver later.")
      sys.exit(0)
    except geni.aggregate.apis.DeleteSliverError:
      pass
    except xmlrpclib.Fault as fault:
      logging.error("[%s] %s" % (fault.faultCode, fault.faultString))
      sys.exit(1)

  logging.info("Building request")
  r = buildRequest(opts)
  r.writeXML("request.xml")
  logging.info("Request saved at request.xml")

  logging.info("Calling createsliver")
  m = am.createsliver(context, opts.slice, r)
  m.writeXML("manifest.xml")
  logging.info("Manifest saved at manifest.xml")

  with open("topo.dot", "w+") as f:
    f.write(geni.util.builddot([m]))
  logging.info("Dotfile saved at topo.dot")

  logging.info("Waiting for sliver ready")
  while True:
    logging.info("...sleeping 15 seconds...")
    time.sleep(15)
    ss = am.sliverstatus(context, opts.slice)
    if ss["geni_status"] == "ready":
      break

  with open("status.json", "w+") as f:
    f.write(json.dumps(ss))
  logging.info("SliverStatus saved at status.json")

  uhgeni.ssh.writeSliceConfig(opts.slice, m, ss, path = opts.config_path, user = opts.user)
  logging.info("ssh config file saved at %s" % (opts.config_path))

  builder = XC.ClientFactory(logging, ssh_cfg_path = "expconfig", manifests = [m], context=context)
  nodes = []
  nodes.append(builder.build("client"))
  nodes.append(builder.build("server"))
  nodes.append(builder.build("monitor"))

  for node in nodes:
    logging.info("[%s] Enabling root ssh and installing mercurial" % (node.host))
    node.ssh.run("echo $(cat ~/.ssh/authorized_keys|grep -v '#') | sudo tee -a /root/.ssh/authorized_keys")
    node.ssh.run("sudo apt-get update && sudo apt-get install -y --no-install-recommends mercurial", timeout = 90)

  lines = open(opts.config_path, "rb").readlines()
  newlines = []
  for line in lines:
    if not line.count("User "):
      newlines.append(line)
      continue
    
    newlines.append("  User root\n")

  with open(opts.config_path, "w+") as f:
    f.writelines(newlines)
  logging.info("Wrote %s with updated users" % (opts.config_path))
